/*******************************************************************************
  Company:
    Microchip Technology Inc.

  File Name:
    telnet.c

  Summary:
    Module for Microchip TCP/IP Stack
     -TelnetT[rXTCP|[g23ԂŒ񋟂
     -Reference: RFC 854

  Description:
    TELNET T[o[

 *******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) <2014> released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*
* ============================================================================ *
* VERSION DATE       BY                   CHANGE/COMMENT                       *
* ---------------------------------------------------------------------------- *
*   1.00  2017-05-08 ޒH[(ނ) f{[hp̂ŏB     *
*   1.10  2017-05-30 ޒH[(ނ) 18F2xK22/24EP256MC202ł̓mFB  *
*
 *******************************************************************************/
//DOM-IGNORE-END

#define __TELNET_C_

#include "tcpip/tcpip.h"

#if defined(STACK_USE_TELNET_SERVER)

// "tcpip_config.h"ŃI[o[ChȂꍇ́A\p[^̃ftHgݒ肵܂B
#if !defined(TELNET_PORT)
// Unsecured Telnet port
#define TELNET_PORT  23
#endif
#if !defined(TELNETS_PORT)
// SSLZLATelnet|[giSTACK_USE_SSL_SERVER`̏ꍇ͖܂j
#define TELNETS_PORT  992
#endif

#if !defined(MAX_TELNET_CONNECTIONS)
// Telnetڑ̍ő吔
#define MAX_TELNET_CONNECTIONS  (3u)
#endif
#if !defined(TELNET_USERNAME)

// Default Telnet user name
#define TELNET_USERNAME  "admin"
#endif
#if !defined(TELNET_PASSWORD)
// Default Telnet password
#define TELNET_PASSWORD  "microchip"
#endif

// Demo title string
static ROM uint8_t strTitle[] = "\x1b[2J\x1b[31m\x1b[1m"    // 2J=ʏ, 31m=ԕ, 1m=
        "\x1b[1;1fMicrochip Telnet Server 1.1\x1b[0m\r\n"   // 0m=Zbg
        "(for this demo, type 'admin' for the login and 'microchip' for the password.)\r\n"
        "Login: ";
// Demo password
// DO Suppress Local Echo (stop telnet client from printing typed characters)
// "\xff\xfd\x2d"̓[JGR[~ׂ̃P[R[h̗l肭삵ĂȂ
static ROM uint8_t strPassword[] = "Password: \xff\xfd\x2d";
// Access denied message
static ROM uint8_t strAccessDenied[] = "Access denied\r\n";
// Successful authentication message
static ROM uint8_t strAuthenticated[] = "Logged in successfully !!\r\n";
// input a command message
static ROM uint8_t strCommand[] = "\r\nPress 'q' to quit "
        "Or input a command(STATE/LED ON/LED OFF)\r\n" ;
// Button status message
static ROM uint8_t strButton[2][16] = {"Button: ON   \r\n",
                                       "Button: OFF  \r\n" } ;
// Command accepted message
static ROM uint8_t strAccepte[] = "I accepted !\r\n" ;
// Invalid command message
static ROM uint8_t strNoCommand[] = "Invalid command !!\r\n" ;
// Demo disconnection message
static ROM uint8_t strGoodBye[] = "\r\nGoodbye!\r\n";

extern APP_CONFIG AppConfig;

/*******************************************************************************
*  ans = adconv() AiOl̓͏s                                   *
*    ans : ǂݍ񂾃AiOlԂ(O`POQR͈̔)                      *
*******************************************************************************/
unsigned int adconv()
{
     unsigned int temp;

#if defined (__XC8)
     GO_nDONE = 1 ;                     // PICɃAiOlǎJnw
     while(GO_nDONE) ;                  // PICǎ芮܂ő҂
     temp = ADRESH ;                    // PIC͓ǎlADRESHADRESL̃WX^[ɃZbg
     temp = ( temp << 8 ) | ADRESL ;    // 10rbg̕\͂ł
#elif defined(__dsPIC33E__)|| defined(__PIC24E__)
     AD1CON1bits.SAMP = 1;              // TvO̊Jn
     Delay10us(1) ;                     // TvȌI܂ő҂
     AD1CON1bits.SAMP = 0 ;             // ϊ̊Jn
     while (!AD1CON1bits.DONE) ;        // ϊ̏I܂ő҂
     temp = ADC1BUF0 ;                  // AiOl(AN0)ǂݏo
#else
     #error telenet.c : The MCU does not support AD conversion.
#endif
     return temp ;
}
/*********************************************************************
 * Function:        void TelnetTask(void)
 *
 * PreCondition:    Stack is initialized()
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        TelnetT[o[֘Ã^XNs܂B
 *                  TelnetXe[g}VƏԃgbLOϐ܂܂܂B
 *
 * Note:            None
 ********************************************************************/
void TelnetTask(void)
{
    uint8_t i;
    uint8_t vTelnetSession;
    uint16_t w, w2;
    TCP_SOCKET MySocket;
    static TICK t ;
    char   buff[20] ;

    enum {
        SM_HOME = 0,
        SM_PRINT_LOGIN,
        SM_GET_LOGIN,
        SM_GET_PASSWORD,
        SM_GET_PASSWORD_BAD_LOGIN,
        SM_PRINT_COMMAND,
        SM_GET_COMMAND,
        SM_DISCONNECT
    } TelnetState ;

    static TCP_SOCKET hTelnetSockets[MAX_TELNET_CONNECTIONS];
    static uint8_t vTelnetStates[MAX_TELNET_CONNECTIONS];
    static bool bInitialized = false;

    // d1̏s
#if defined(WF_CS_TRIS)
    if ((!bInitialized) || (AppConfig.hibernateFlag)) {
#else
    if (!bInitialized) {
#endif
        for (vTelnetSession = 0; vTelnetSession < MAX_TELNET_CONNECTIONS; vTelnetSession++) {
            hTelnetSockets[vTelnetSession] = INVALID_SOCKET;
            vTelnetStates[vTelnetSession] = SM_HOME;
        }
        bInitialized = true;
    }

    // etelnetZbVƏXe[g̕ύXTX/RXf[^[v
    for (vTelnetSession = 0; vTelnetSession < MAX_TELNET_CONNECTIONS; vTelnetSession++) {
        // ̃ZbV̏Xe[g[h
        MySocket    = hTelnetSockets[vTelnetSession];
        TelnetState = vTelnetStates[vTelnetSession];

        // [gNCAgؒfꍇ́AԂZbgĉB
        if (MySocket != INVALID_SOCKET) {
            if (TCPWasReset(MySocket))
                TelnetState = SM_PRINT_LOGIN;
        }

        // ZbVԂ
        switch (TelnetState) {
           /*** \Pbg[gTCPT[o[ɐڑ鏈 ***/
           case SM_HOME :
               MySocket = TCPOpen(0, TCP_OPEN_SERVER, TELNET_PORT, TCP_PURPOSE_TELNET);

               // TCP\PbgȂꍇA𒆎~܂B
               // type TCP_PURPOSE_TELNET is available If this ever happens, you need to go add one to tcpip_config.h
               if (MySocket == INVALID_SOCKET)  break ;

               // SSLT[o[̃T|[gLȏꍇ́ASSLXi[J܂B
#if defined(STACK_USE_SSL_SERVER)
               TCPAddSSLListener(MySocket, TELNETS_PORT);
#endif
               // Xe[g(SM_PRINT_LOGIN)ɐi߂
               TelnetState++;
               break;

           /*** ^Cg̕f[^𑗐M鏈 ***/
           case SM_PRINT_LOGIN:
#if defined(STACK_USE_SSL_SERVER)
               // TELNET_REJECT_UNSECURED`ĂꍇASłȂڑۂ
   #if defined(TELNET_REJECT_UNSECURED)
               if (!TCPIsSSL(MySocket)) {
                   if (TCPIsConnected(MySocket)) {
                       TCPDisconnect(MySocket);
                       TCPDisconnect(MySocket);
                       break;
                   }
               }
   #endif
               // ܂nhVFCNĂꍇAMȂŉB
               if (TCPSSLIsHandshaking(MySocket)) break ;
#endif

               // ^Cg̕\Pbgɏގo邩ǂmF
               if (TCPIsPutReady(MySocket) < strlenpgm((ROM char *) strTitle)) break ;
               // ^Cgf[^𑗐Mobt@ɓ܂B
               TCPPutROMString(MySocket, strTitle);
               // pPbg𑗐M
               TCPFlush(MySocket);
               // Xe[g(SM_GET_LOGIN)ɐi߂
               TelnetState++;

           /*** ͂ꂽOC𒲂ׂ鏈 ***/
           case SM_GET_LOGIN:
               // pX[hv̕\Pbgɏގo邩ǂmF
               if (TCPIsPutReady(MySocket) < strlenpgm((ROM char *) strPassword)) break ;
               // [U[^[('LF')ǂmF
               w = TCPFind(MySocket, '\n', 0, false);  // 'CR+LF''LF'ŔfĂ
               if (w == 0xFFFFu) {
                   // łȂ
                   if (TCPGetRxFIFOFree(MySocket) == 0u) {
                       // Mobt@tԂȂA^Cg\̏܂Ŗ߂
                       TelnetState = SM_PRINT_LOGIN;
                       TCPGetArray(MySocket, NULL, w + 1);
                   }
                   break;
               }

               // OC - 啶ƏʂȂ
               w2 = TCPFindROMArray(MySocket, (ROM uint8_t *) TELNET_USERNAME, sizeof (TELNET_USERNAME) - 1, 0, true);
               if (!((sizeof (TELNET_USERNAME) - 1 == w - w2) || (sizeof (TELNET_USERNAME) == w - w2))) {
                   // [U[܂łB
                   // A[U[̐RȂ悤ɂĂ݂܂傤
                   TelnetState = SM_GET_PASSWORD_BAD_LOGIN;
               } else {
                   TelnetState = SM_GET_PASSWORD;
               }
               // OCmFꂽ̂ŁAFIFOobt@ɂ܂
               TCPGetArray(MySocket, NULL, w + 1);

               // pX[hv̕𑗐M
               TCPPutROMString(MySocket, strPassword);
               TCPFlush(MySocket);
               break;

           /*** ͂ꂽpX[h𒲂ׂ鏈 ***/
           case SM_GET_PASSWORD:
           case SM_GET_PASSWORD_BAD_LOGIN:
               // ANZXF؂̕\Pbgɏގo邩ǂmF
               if (TCPIsPutReady(MySocket) < strlenpgm((ROM char *) strAuthenticated)) break ;
               // [U[^[ǂmF
               w = TCPFind(MySocket, '\n', 0, false);
               if (w == 0xFFFFu) {
                   if (TCPGetRxFIFOFree(MySocket) == 0u) {
                        // Mobt@tԂȂA^Cg\̏܂Ŗ߂
                        TelnetState = SM_PRINT_LOGIN;
                        TCPGetArray(MySocket, NULL, w + 1);
                   }
                   break;
               }

               // pX[h - 啶Əʂ
               w2 = TCPFindROMArray(MySocket, (ROM uint8_t *) TELNET_PASSWORD, sizeof (TELNET_PASSWORD) - 1, 0, false);
               if ((w2 == 0xFFFF) || (TelnetState == SM_GET_PASSWORD_BAD_LOGIN)) {
                   // pX[h܂łA̓OCقȂ
                   // ^Cg\̏܂Ŗ߂
                   TCPGetArray(MySocket, NULL, w + 1);
                   TelnetState = SM_PRINT_LOGIN;
                   break;
               }
               // pX[hmFꂽ̂ŁAFIFOobt@ɂ܂
               TCPGetArray(MySocket, NULL, w + 1);

               // ANZXF؂̕𑗐M
               TCPPutROMString(MySocket, strAuthenticated);
               TCPFlush(MySocket);
               TelnetState = SM_PRINT_COMMAND;
               // No break

           /*** R}hv̕\鏈 ***/
           case SM_PRINT_COMMAND:
               // R}hv̕\Pbgɏގo邩ǂmF
               if (TCPIsPutReady(MySocket) < strlenpgm((ROM char *) strCommand)) break ;
               //  R}hṽf[^𑗐M
               TCPPutROMString(MySocket, strCommand);
               TCPFlush(MySocket);
               // Xe[g(SM_GET_COMMAND)ɐi߂
               TelnetState++;

           /*** ͂ꂽR}hɉ ***/
           case SM_GET_COMMAND:
               // \̕\Pbgɏގo邩ǂmF
               if (TCPIsPutReady(MySocket) < strlenpgm((ROM char *) strButton)) break ;
               // [U[^[('LF')ǂmF
               w = TCPFind(MySocket, '\n', 0, false);  // 'CR+LF''LF'ŔfĂ
               if (w == 0xFFFFu) {
                   // łȂ
                   if (TCPGetRxFIFOFree(MySocket) == 0u) {
                       // Mobt@tԂȂA^Cg\̏܂Ŗ߂
                       TelnetState = SM_PRINT_COMMAND;
                       TCPGetArray(MySocket, NULL, w + 1);
                   }
                   break;
               }

               // Mobt@ǂݎf[^𓾂
               if (TCPIsGetReady(MySocket)) {
                   // "STATE"̌
                   w2 = TCPFindROMArray(MySocket,(ROM uint8_t *)"STATE", 5, 0, false);
                   if (w2 != 0xFFFFu) {
                       // AN1(ϒR)̒lǂݎ
                       sprintf(buff,"\r\nAnalog: %04d\r\n",adconv()) ;
                       TCPPutROMString(MySocket, (ROM uint8_t *)buff);
                       // XCb`̏Ԃ𓾂
                       if (BUTTON0_IO == 0) TCPPutROMString(MySocket, (ROM uint8_t *)&strButton[0]);
                       else                 TCPPutROMString(MySocket, (ROM uint8_t *)&strButton[1]);
                       // [gTCPT[o[ɑ
                       TCPFlush(MySocket);
                       TelnetState = SM_PRINT_COMMAND;
                       TCPGetArray(MySocket, NULL, w + 1);
                       break ;
                   }
                   // "LED"̌
                   w2 = TCPFindROMArray(MySocket,(ROM uint8_t *)"LED", 3, 0, false);
                   if (w2 != 0xFFFFu) {
                       w2 = TCPFindROMArray(MySocket,(ROM uint8_t *)"ON", 2, 0, false);
                       if (w2 != 0xFFFFu) {
                            // v󂯕t܂̃bZ[W𑗐M
                            TCPPutROMString(MySocket, (ROM uint8_t *)strAccepte);
                            TCPFlush(MySocket);
                            TelnetState = SM_PRINT_COMMAND;
                            TCPGetArray(MySocket, NULL, w + 1);
                            LED0_IO = 1 ;    // LED_
                            break ;
                       }
                       w2 = TCPFindROMArray(MySocket,(ROM uint8_t *)"OFF", 3, 0, false);
                       if (w2 != 0xFFFFu) {
                            // v󂯕t܂̃bZ[W𑗐M
                            TCPPutROMString(MySocket, (ROM uint8_t *)strAccepte);
                            TCPFlush(MySocket);
                            TelnetState = SM_PRINT_COMMAND;
                            TCPGetArray(MySocket, NULL, w + 1);
                            LED0_IO = 0 ;    // LED
                            break ;
                       }
                   }

                   // PoCgǂݍ
                   TCPGet(MySocket, &i);
                   switch (i) {
                   case 'q':
                   case 'Q':
                       if (TCPIsPutReady(MySocket) >= strlenpgm((ROM char *)strGoodBye)+4) {
                            // IbZ[W𑗂A1b̐ؒf҂
                            TCPGetArray(MySocket, NULL, w + 1);
                            TCPPutROMString(MySocket, (ROM uint8_t *) "\x1b[1m");
                            TCPPutROMString(MySocket, strGoodBye);
                            TCPFlush(MySocket);
                            t = TickGet() ;
                            TelnetState = SM_DISCONNECT;
                            break;
                       }
                   default:
                       TCPGetArray(MySocket, NULL, w + 1);
                       // ͂ꂽR}h͖惁bZ[W𑗂
                       TCPPutROMString(MySocket, strNoCommand) ;
                       TCPFlush(MySocket);
                       break ;
                   }
               }
               break;

           /*** 1bɐؒf ***/
           case SM_DISCONNECT:
               if(TickGet() - t >= TICK_SECOND) {
                   // ڑ؂^Cg\ɖ߂
                   TCPDisconnect(MySocket);
                   TelnetState = SM_PRINT_LOGIN;
               }
               break ;
        }

        // ݂̃ZbVԂ̃Xe[gL^
        hTelnetSockets[vTelnetSession] = MySocket;
        vTelnetStates[vTelnetSession]  = TelnetState;
    }
}

#endif // defined(STACK_USE_TELNET_SERVER)
